////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "bxdfbase.h"
#include "bxdforennayar.h"

CBxDFOrenNayar::CBxDFOrenNayar(float Sig /* = 45.0 */) : CBxDFBase( BSDF_DIFFUSE  )
{
  SetSigma( Sig );
}

CBxDFOrenNayar::~CBxDFOrenNayar(void)
{
}

void CBxDFOrenNayar::SetSigma( float Val )
{
  float Sigma = (Val * PI / 180.0);
  float Sigma2 = Sigma * Sigma;
  m_A = 1.f - (Sigma2 / (2.f * (Sigma2 + 0.33f)));
  m_B = 0.45f * Sigma2 / (Sigma2 + 0.09f);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CBxDFOrenNayar::f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade /*= 1.0*/ )

////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Shade ); UNUSED( Power ); UNUSED( LightColor ); UNUSED( RenderMat );
  Vector InDir  = In.dir();
  Vector OutDir = -Out.dir();

  double CosThetaI = Normal.dot( InDir );
  double CosThetaO = Normal.dot( OutDir );
  double SinThetaI = sqrt( std::max( 0.0, 1.0 - CosThetaI * CosThetaI ) );
  double SinThetaO = sqrt( std::max( 0.0, 1.0 - CosThetaO * CosThetaO ) );

  // Compute cosine term of Oren-Nayar model
  double SinPhiI = ( InDir.y  / SinThetaI), CosPhiI = ( InDir.x  / SinThetaI);
  double SinPhiO = ( OutDir.y / SinThetaO), CosPhiO = ( OutDir.x / SinThetaO);

  double dcos = CosPhiI * CosPhiO + SinPhiI * SinPhiO;
  double MaxCos = std::max( 0.0, dcos );
  // Compute sine and tangent terms of Oren-Nayar model
  double SinAlpha, TanBeta;
  if (fabs( CosThetaI ) > fabs( CosThetaO )) {
    SinAlpha = SinThetaO;
    TanBeta = SinThetaI / fabs( CosThetaI );
  }
  else {
    SinAlpha = SinThetaI;
    TanBeta = SinThetaO / fabs( CosThetaO );
  }

  double RetCol = std::min( 1.0, 1.0 / PI * (m_A + m_B * MaxCos * SinAlpha * TanBeta));
  return RetCol;
}
// vim:ts=2
