////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//
// BxDF for standard Phong model shading
//

#pragma once
#ifndef BXDF_PHONG_H
#define BXDF_PHONG_H

#include "bxdfbase.h"

class CBxDFBase;
class CFresnelDiel;

class CBxDFPhong : public CBxDFBase
{
public:
	CBxDFPhong();
	virtual ~CBxDFPhong( void );

	// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
	virtual double f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor = Color( 1.0, 1.0, 1.0), double Power = 1.0, float Shade = 1.0f ) override;

	// Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
	// BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
	// continue recursively.
	virtual double Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0f ) override;
	virtual double GetPDF( const Vector& Out, const Vector& In, const Vector& Normal ) const override {
		UNUSED( In ); UNUSED( Out ); UNUSED( Normal );
		return 1.0; };

protected:
	virtual double Sample_f_Reflection( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr );
	virtual double Sample_f_Transmission( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr );

protected:

	CFresnelDiel* m_pFresnelObj;
};

#endif
// vim:ts=2
