////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "matblinn.h"

//
// Blinn material which approximates the microfacet distribution. Thus, it can be controlled easier as the absorption, roughness etc. does not need to be pre-specified
//

CMatBlinn::CMatBlinn( CBaseIntegrator* pInt ) : CPhysMatInterface( pInt, static_cast<CBxDFBase::BxDFType>( CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_SPECULAR | CBxDFBase::BSDF_REFLECTION | CBxDFBase::BSDF_TRANSMISSION ) , false /* false = no MC sampling required */ )
{
  m_pFresnel = new CFresnelDiel;
  m_pBxDFReflection = new CBxDFSpecReflection( m_pFresnel );
}

CMatBlinn::~CMatBlinn(void)
{
  delete m_pFresnel;
  delete m_pBxDFReflection;
}

///////////////////////////////////////////////////////////////////////////////////////////////

double CMatBlinn::f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade /*= 1.0*/ )

///////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );

  double Diffuse  = m_BxDFDiffuse.f( Out, In, Normal, RenderMat, LightColor, Shade );
  double Specular = m_BxDFSpecular.f( Out, In, Normal, RenderMat, LightColor, Shade );

  return Diffuse + Specular;
}

///////////////////////////////////////////////////////////////////////////////////////////////

double CMatBlinn::Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, CBxDFBase::BxDFType Type, const Color& LightColor /*= Color( 1.0, 1.0, 1.0 )*/, double Power /*= 1.0*/, float Shade /*= 1.0*/ )

///////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );

  if( Type == CBxDFBase::BSDF_REFLECTION )
    return m_pBxDFReflection->Sample_f( Out, In, Pdf, Normal, RenderMat, pRandNr, Type, LightColor, Shade );
  else if( Type == CBxDFBase::BSDF_TRANSMISSION )
    return m_BxDFTransmission.Sample_f( Out, In, Pdf, Normal, RenderMat, pRandNr, Type, LightColor, Shade );

  return CPhysMatInterface::Sample_f( Out, In, Pdf, Normal, RenderMat, pRandNr, Type, LightColor, Shade );
}
// vim:ts=2
