////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "matfresnelblend.h"
#include "bxdffresnelblendex.h"

CMatFresnelBlend::CMatFresnelBlend( CBaseIntegrator* pInt, double eX /* = 4 */, double eY /* = 20 */, bool bPerfectSpecularRefl /*= false*/ ) : CPhysMatInterface( pInt, static_cast<CBxDFBase::BxDFType>( CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_SPECULAR | CBxDFBase::BSDF_REFLECTION ), true /* false = no MC sampling required */ )
{
  m_pMicroDist = new CAnisotropicMicroDist( eX, eY);

  if( !bPerfectSpecularRefl)
    m_pBxDFDiffuse = new CBxDFFresnelBlend( m_pMicroDist );
  else
    m_pBxDFDiffuse = new CBxDFFresnelBlendEx( m_pMicroDist );
}

CMatFresnelBlend::CMatFresnelBlend( CBaseIntegrator* pInt, CMicrofacetDistribution* pMicroDist, bool bPerfectSpecularRefl /*= false*/ ) : CPhysMatInterface( pInt, static_cast<CBxDFBase::BxDFType>(CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_SPECULAR | CBxDFBase::BSDF_REFLECTION), true /* false = no MC sampling required */ ),
                  m_pMicroDist(pMicroDist)
{
	if(!bPerfectSpecularRefl)
		m_pBxDFDiffuse = new CBxDFFresnelBlend( m_pMicroDist );
	else
		m_pBxDFDiffuse = new CBxDFFresnelBlendEx( m_pMicroDist );
}


CMatFresnelBlend::~CMatFresnelBlend(void)
{
  delete m_pMicroDist;
  delete m_pBxDFDiffuse;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CMatFresnelBlend::f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade /*= 1.0*/ )

// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
	UNUSED( Power );
	// return phong for diffuse reflection
	return m_pBxDFDiffuse->f( Out, In, Normal, RenderMat, LightColor, Shade );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CMatFresnelBlend::Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, CBxDFBase::BxDFType Type, const Color& LightColor /*= Color( 1.0, 1.0, 1.0 )*/, double Power /*= 1.0*/, float Shade /*= 1.0*/ )

// Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
// BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
// continue recursively.
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
	UNUSED( Power );

  if( Type == CBxDFBase::BSDF_REFLECTION || Type == CBxDFBase::BSDF_DIFFUSE )
    return m_pBxDFDiffuse->Sample_f( Out, In, Pdf, Normal, RenderMat, pRandNr, Type, LightColor, Shade);

  return CPhysMatInterface::Sample_f( Out, In, Pdf, Normal, RenderMat, pRandNr, Type, LightColor, Shade );
}
// vim:ts=2
