////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
/// Authors: Christian.Theis@cern.ch
//			     Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once
#ifndef MAT_PHONG_H
#define MAT_PHONG_H

#include "physmatinterface.h"
#include "bxdfphong.h"
#include "baseintegrator.h"

class CMatPhong : public CPhysMatInterface
{
public:
	CMatPhong( CBaseIntegrator* pInt ) : CPhysMatInterface( pInt, CBxDFBase::BSDF_ALL, false /* false = no MC sampling required */ ) {};
	virtual ~CMatPhong( void ) {};

	// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
	virtual double f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade = 1.0f ) override {
		UNUSED( Power );
		return m_BxDF.f( Out, In, Normal, RenderMat, LightColor, Shade );
	}

	// Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
	// BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
	// continue recursively.
	virtual double Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, CBxDFBase::BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0f ) override {
		UNUSED( Power );
		return m_BxDF.Sample_f( Out, In, Pdf, Normal, RenderMat, pRandNr, Type, LightColor, Shade );
	}

protected:
	CBxDFPhong m_BxDF;
};

#endif
// vim:ts=2
