////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "mattranslucent.h"

CMatTranslucent::CMatTranslucent( CBaseIntegrator* pInt ) : CPhysMatInterface( pInt, static_cast<CBxDFBase::BxDFType>( CBxDFBase::BSDF_DIFFUSE | CBxDFBase::BSDF_SPECULAR | CBxDFBase::BSDF_TRANSMISSION | CBxDFBase::BSDF_REFLECTION ), true /* false = no MC sampling required */ )
{
  m_pBxDFDiffuse      = new CBxDFPhong;
  m_pBxDFTranslucent  = new CBxDFTranslucent; // specular transmission
}

CMatTranslucent::~CMatTranslucent(void)
{
  delete m_pBxDFDiffuse;
  delete m_pBxDFTranslucent;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CMatTranslucent::f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade /*= 1.0*/ )

// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );
  // return phong for diffuse reflection
  return m_pBxDFDiffuse->f( Out, In, Normal, RenderMat, LightColor, Shade );
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CMatTranslucent::Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, double* pRandNr, CBxDFBase::BxDFType Type, const Color& LightColor /*= Color( 1.0, 1.0, 1.0 )*/, double Power /*= 1.0*/, float Shade /*= 1.0*/ )


// Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
// BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
// continue recursively.
////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( Power );

  if( Type == CBxDFBase::BSDF_REFLECTION )
    return m_pBxDFDiffuse->Sample_f( Out, In, Pdf, Normal, RenderMat, pRandNr, Type, LightColor, Shade);
  else if( Type == CBxDFBase::BSDF_TRANSMISSION )
    return m_pBxDFTranslucent->Sample_f( Out, In, Pdf, Normal, RenderMat, pRandNr, Type, LightColor, Shade);

  return CPhysMatInterface::Sample_f( Out, In, Pdf, Normal, RenderMat, pRandNr, Type, LightColor, Shade );
}
// vim:ts=2
