#ifndef MICROFACET_DIST_H
#define MICROFACET_DIST_H

#include "math/matrix.h"

class CMicrofacetDistribution {
public:
  // MicrofacetDistribution Interface
  CMicrofacetDistribution();
  virtual ~CMicrofacetDistribution ();
  virtual double D(const Vector& wh, const Vector& Normal ) const = 0;
  virtual void Sample_f( const Vector& Out, Vector& In, const Vector& Normal, double u1, double u2, float& Pdf ) const = 0;
  virtual float GetPDF(const Vector& Out, const Vector& In, const Vector& Normal ) const = 0;

	// standard normalization function
	virtual double G( const Vector& Out, const Vector& In, const Vector& wh, const Vector& Normal ) const {
		double NdotWh = fabs( (Normal.dot( wh )) );
		double NdotWo = fabs( (Normal.dot( Out )) );
		double NdotWi = fabs( (Normal.dot( In )) );
		double WOdotWh = fabs( Out.dot( wh ) );
		return (std::min<double>( 1.0, std::min<double>( (2.0 * NdotWh * NdotWo / WOdotWh), (2.0 * NdotWh * NdotWi / WOdotWh) ) ));
	}

};

#endif
// vim:ts=2
