/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	10-Feb-2018
 */

#ifndef __GEOMETRY_BOXBODY_H
#define __GEOMETRY_BOXBODY_H

#include "gbody.h"

// ================================= GBOXBody ==================================
class GBOXBody : public GBody {
public:
	GBOXBody(const std::string& aname, const Type atype=Type::BOX)
		: GBody(aname, atype) {}
	void	set(const double x,  const double y,  const double z,
		    const double ux, const double uy, const double uz,
		    const double vx, const double vy, const double vz,
		    const double wx, const double wy, const double wz);
	void	set(const Vector& pos, const Vector& u,
		    const Vector& v,   const Vector& w)
			{ set(pos.x, pos.y, pos.z,
			      u.x, u.y, u.z,
			      v.x, v.y, v.z,
			      w.x, w.y, w.z); }
	void	set(const double xmin, const double xmax,
		    const double ymin, const double ymax,
		    const double zmin, const double zmax);
	virtual	void	set(int n, double *);
	virtual	int	get(double *) const;
	virtual	int	nodes()		const	{ return type()==Type::BOX?4:1; }
	virtual	void	position(const Point& r);
	virtual	void	rotate(const double angle, const Vector& axis);
	virtual void	move(int item, const Point& r, const Vector& w);

// FIXME
//	virtual	double	distance(const double x, const double y, const double z) const;

protected:
	virtual	void	createQuads();
	virtual void	createMesh();
	virtual	void	_updateBBox();
#ifdef USE_OBBOX
	virtual	void	_updateOBB(bool in=false);
#endif
	virtual	Location _locationWrt(const GBody* body) const;
}; // GBOXBody

// ================================= GWEDBody ==================================
class GWEDBody : public GBody {
public:
	GWEDBody(const std::string& aname) : GBody(aname, Type::WED) {}

	Vector	N()	const;

	void	set(const double x,  const double y,  const double z,
		    const double ux, const double uy, const double uz,
		    const double vx, const double vy, const double vz,
		    const double wx, const double wy, const double wz);
	void	set(const Vector& pos, const Vector& u,
		    const Vector& v,   const Vector& w)
			{ set(pos.x, pos.y, pos.z,
			      u.x, u.y, u.z,
			      v.x, v.y, v.z,
			      w.x, w.y, w.z); }
	virtual	void	set(int n, double *);
	virtual	int	get(double *) const;
	virtual	int	nodes()		const	{ return 4; }
	virtual void	move(int item, const Point& r, const Vector& w);

// FIXME
//	virtual	double	distance(const double x, const double y, const double z) const;

protected:
	virtual	void	createQuads();
	virtual void	createMesh();
	virtual	void	_updateBBox();
#ifdef USE_OBBOX
	virtual	void	_updateOBB(bool in=false);
#endif
}; // GWEDBody

#endif
