/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	01-Apr-2011
 */

#ifndef __GEOMETRY_VZONE_H
#define __GEOMETRY_VZONE_H

#include "gzone.h"
#include "vregion.h"

#include "math/vector.h"

class GBody;
class CBody;
class VBody;
class ViewPort;
class GeometryEngine;
#ifdef USE_OBBOX
class OBBox;
#endif

// =================================== VZone ===================================
class VZone {
private:
	GZone*	 _zone;		// Zone definition
	VRegion* _region;	// parent region
	int	 _generation;	// Generation number, to be checked for
				// validity against the _zone generation
	bool	_location;	// true if "in" is valid
				// FIXME merge _location and _in to Location
	bool	_in;		// if window is inside or outside the region
				// condition to be checked ONLY if location=true

public:
	VZone(GZone* z=NULL, VRegion* r=NULL)	{ init(z,r); }
	void	init(GZone* z=NULL, VRegion* r=NULL);

	bool	valid()		const { return _generation == _zone->generation(); }
	bool	invalid()	const { return _generation != _zone->generation(); }
	void	setInvalid()          { _generation = -1;  }
	void	setValid()	      { _generation = _zone->generation(); }

	GZone*	zone()		const { return _zone; }
	VRegion* region()	const { return _region; }
const	GRegion* gregion()	const { return _region->region(); }
const	std::string&	name()	const { return _region->name(); }
	int	id()		const { return _zone->id(); }

	int	size()		const { return _zone->size();  }
	bool	std()		const { return _zone->std();   }
	bool	rpn()		const { return _zone->rpn();   }
const	GBody*	gexpr(int i)	const { return _zone->expr[i]; }
inline	VBody*	vbody(const GBody* body) const;

	// location operations
	// inside with p,v
	bool	inside2D(GeometryEngine* engine,
			 const double x, const double y, const double z,
			 const double u, const double v, const double w) const;
	bool	inside2D(GeometryEngine* engine,
			  const Vector& p, const Vector& d) const
			{ return inside2D(engine, p.x,p.y,p.z, d.x,d.y,d.z); }

	// inside & distance with a p,v,t
	bool	inside(GeometryEngine* engine,
			 const double x, const double y, const double z,
			 const double u, const double v, const double w,
			 const double t) const;
	bool	inside(GeometryEngine* engine,
			  const Vector& p, const Vector& d, double t) const
			{ return inside(engine, p.x,p.y,p.z, d.x,d.y,d.z, t); }

	CBody*	distance(GeometryEngine* engine,
			     const double x, const double y, const double z,
			     const double u, const double v, const double w,
			     double* tmin, const double tmax) const;
	CBody*	distance(GeometryEngine* engine,
			     const Vector& p, const Vector& d,
			     double* tmin, const double tmax) const
			{ return distance(engine, p.x,p.y,p.z, d.x,d.y,d.z, tmin, tmax); }

	void	updateLocation(const ViewPort& view);
	bool	ignore()	const { return _location && !_in; }
	bool	location()	const { return _location; }
	bool	in()		const { return _in; }

	// bounding boxes
const	BBox&	bbox()		const { return _zone->bbox(); };
#ifdef USE_OBBOX
const	OBBox&	obbox()		const { return _zone->obbox(); };
#endif

	size_t	memory()	const { return sizeof(VZone); }

private:
inline	static CBody* cbody(GeometryEngine* engine, const GBody* body);
}; // VZone

#endif
