/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	13-Jun-2018
 */

#ifndef __GRAPH_BFONT_H
#define __GRAPH_BFONT_H

#include <string>

#include "os.h"

class Painter;

/* BFont class */
class BFont {
private:
	std::string	_name;		/** font name			*/
	int	imageWidth;		/** image width			*/
	int	imageHeight;		/** image height		*/
	int	_width;			/** fixed width of each char	*/
	int	_height;		/** height of each char		*/
	byte*	imageData;		/** Font image data		*/
	byte	charLength[256];	/** var-width of each character	*/

public:
	BFont() : _width(-1), _height(-1), imageData(NULL) {}
	BFont(const char *filename) :
			_width(-1),
			_height(-1),
			imageData(NULL)
			{ load(filename); }
	~BFont() { clean(); }

const	std::string& name()	const	{ return _name; }
const	char*	cname()		const	{ return _name.c_str(); }
	int	width()		const	{ return _width; }
	int	height()	const	{ return _height; }

	bool	load(const char *filename);
	void	set(const char* n, int w, int h, byte* data);
	void	set(const char* n, int w, int h, dword* data);

	int	draw(Painter& painter, int x, int y, const dword color, const byte ch);
	int	drawScaled(Painter& painter, int x, int y, const dword color,
			const byte ch, const int scale);
	int	vdraw(Painter& painter, int x, int y, const dword color, const byte ch);
	int	vdrawScaled(Painter& painter, int x, int y, const dword color,
			const byte ch, const int scale);
	int	drawOutline(Painter& painter, int x, int y,
			const dword color, const dword outline, const byte ch);

	int	measure(const char *str);

protected:
	byte*	charPtr(const int ch) const
			{ return imageData + ((ch>>4)*_height*imageWidth) + (ch&0x0F)*_width; }
	void	build();
	void	clean();
}; // BFont

#endif
