/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * BasedOn:	Chris.Theis@cern.ch
 * Date:	unknown
 *
 */
#ifndef __GRAPH_DXFEXPORT_H
#define __GRAPH_DXFEXPORT_H

#include <iomanip>
#include "exportbase.h"
#include "math/vector.h"

class DXFExport : public ExportBase {
public:
	DXFExport(const std::string& filename) : ExportBase(filename) {
			if (_file.is_open()) writeHeader();
			_lastColor = 0;
			_best      = 0;
		}
	virtual ~DXFExport(void) {
			if (_file.is_open()) writeEOF();
		}
	bool line(double x1, double y1, double x2, double y2, dword color=0, const char* layer=NULL);
	bool point(double x, double y, dword color=0, const char* layer=NULL);
	bool circle(double x, double y, double radius, dword color=0, const char* layer=NULL );
	bool arc(double x, double y, double radius, double startPhi, double endPhi, dword color=0,  const char* layer=NULL);
	bool polyline(Path& path, dword color=0, const char* layer=NULL);

protected:
	bool    writeHeader();
	bool    writeEOF();

	void	write(const int id, const int value) {
			_file << std::setw(3) << id << std::endl;
			_file << std::setw(0) << value << std::endl;
		}

	void	write(const int id, const double value) {
			_file << std::setw(3) << id << std::endl;
			_file << std::setw(0) << value << std::endl;
		}

	void	write(const int id, const char* str) {
			_file << std::setw(3) << id << std::endl;
			_file << str << std::endl;
		}

	void	write(const int id, const double x, const double y) {
			write(10+id, x);
			write(20+id, y);
		}

	void	write(const int id, const double x, const double y, const double z) {
			write(10+id, x);
			write(20+id, y);
			write(30+id, z);
		}

	void	write2D(const int id, const Vector& v) {
			write(10+id, v.x);
			write(20+id, v.y);
		}

	void	write(const int id, const Vector& v) {
			write(10+id, v.x);
			write(20+id, v.y);
			write(30+id, v.z);
		}

	void	writeLayer(const char *layer=NULL) {
			if (layer) write(8,layer);
		}

	void	writeColor(const int color) {
			if (color>=0) write(62, closestColor(color));
		}

	int	closestColor(const dword color);

protected:
	dword	_lastColor;
	int	_best;
}; // DXFExport

#endif

