/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 */

#include "palette.h"

#include "color.h"
#include "math/eps.h"

/** reset */
void Palette::reset()
{
	// color scale
	_log         = true;
	_interpolate = false;
	_min         = -4;
	_max         =  4;
	_invert      = false;
	_alphamin    = false;
	_alphamax    = false;
	_n           = 2;
	_palette[0]  = 0;
	_palette[1]  = 0XFFFFFF;
	init();
} // reset

/** init */
void Palette::init()
{
	_n    = Range(2, _n, _MAXCOLORS);
	_step = (_max - _min) / (double)(_interpolate? _n-1 : _n);
} // init

/** checkLimits */
void Palette::checkLimits()
{
	if (_min > _max) {
		Swap(_min, _max);
		_invert = !_invert;
		if (_step < 0.0) _step = -_step;
	}
	if (_step <= SMALL) {
		_step = 1.0;
		_max = _min + 4.0;
	}
} // checkLimits

/** color */
dword Palette::color(const double value) const
{
	if (value<=_min) {
		if (_alphamin) return COLOR_TRANSPARENT;
		return _invert? _palette[_n-1] : _palette[0];
	}
	if (value>=_max) {
		if (_alphamax) return COLOR_TRANSPARENT;
		return _invert? _palette[0] : _palette[_n-1];
	}

	double div = (value-_min) / _step;
	int ic = (int)div;
	if (_invert) {
		div = (double)_n - div;
		ic  = _n - ic - 1;
		if (_interpolate) {
			div--;
			ic--;
		}
	}
	if (_interpolate) {
		int x  = (int)(255.0*(div - (double)ic));
		int x1 = 256 - x;

		Color32 lo, hi;
		if (_invert) {
			Swap(x,x1);
			lo = _palette[ic+1];
			hi = _palette[ic];
		} else {
			lo = _palette[ic];
			hi = _palette[ic+1];
		}

		Color32 col(0);
		col.red()   = (byte)(((int)lo.red()  *x1 + (int)hi.red()  *x)>>8);
		col.green() = (byte)(((int)lo.green()*x1 + (int)hi.green()*x)>>8);
		col.blue()  = (byte)(((int)lo.blue() *x1 + (int)hi.blue() *x)>>8);

		return col;
	} else
		return _palette[ic];
} // color
