/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Set of low discrepancy sampling functions as well as several geometric samplers
 *
 * Author:	Christian.Theis@cern.ch
 * Date:	13-May-2020
 */

#ifndef NIEDERREITER_H
#define NIEDERREITER_H

#include <vector>
#include <iostream>
#include <cassert>

//
// Constants defining the limits of the Niederreiter QMC sequence generation
//
//
//    int MAXDEG, the highest degree of polynomial
//    to be handled.
//
//    int DIM_MAX, the maximum dimension that will be used.
//
//    int NBITS, the number of bits (not counting the sign) in a
//    fixed-point integer.
//
//    double RECIP, the multiplier which changes the
//    integers in NEXTQ into the required real values in QUASI.
//
# define MAXDEG 50
# define DIM_MAX 20
# define NBITS 31

// Helper functions for the generation of a Niederreiter sequence
void    Calcc2( int dimen, long cj[DIM_MAX][NBITS] );
void    Calcv2( long maxv, long px_deg, long px[MAXDEG + 1], long add[2][2], long mul[2][2], long sub[2][2], long* b_deg, long b[MAXDEG + 1], long v[] );
void    Plymul2( long add[2][2], long mul[2][2], long pa_deg, long pa[MAXDEG + 1], long pb_deg, long pb[MAXDEG + 1], long* pc_deg, long pc[MAXDEG + 1] );
void    Setfld2( long add[2][2], long mul[2][2], long sub[2][2] );
double  Niederreiter2( long int* seed );
void    Niederreiter2ND( int dim_num, long int* seed, double* quasi );
void    Niederreiter2Generate( int Dim, long NrOfValues, long int* pSeed, std::vector<double>& Sequence );


#endif

//vim: ts=2
