/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	07-Mar-2016
 */

#ifndef __MATH_SCATTER_H
#define __MATH_SCATTER_H

#include "os.h"
#include "tool/array.h"

//-----------------------------------------------------------------------------
// Pair x,y value
//-----------------------------------------------------------------------------
class XYPair {
public:
	double	x;
	double	y;
public:
	XYPair(const double ax=0.0, const double ay=0.0) : x(ax), y(ay) {}
	XYPair(const XYPair&) = default;
	XYPair&	operator =(const XYPair&) = default;

static	int compare(const XYPair& a, const XYPair& b) { return Cmp(a.x, b.x); }

	// not used but to keep compiler happy
	bool	operator==(const XYPair& p) const	{ return x == p.x; }
};

//=============================================================================
// Scatter data class
//=============================================================================
class Scatter {
private:
	Array<XYPair>	data;
public:
	Scatter() { data.compare(XYPair::compare); }
	~Scatter();

	bool	load(const char* filename);
	bool	isEmpty() const				{ return data.size()==0; }

const	XYPair&	operator[](const int i)	const		{ return data[i]; }

	double	interpolate(const double x) const;
	double	interpolateLog(const double x) const;
	double	operator()(const double x) const	{ return interpolate(x); }
}; // class Scatter

#endif
