/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	11-Jan-2006
 */

#include "stream.h"

#include <assert.h>

/* --- open --- */
bool Stream::open(IO io, bool bin)
{
	_io    = io;
	binary = bin;
	header = false;
	footer = false;
	if (isOpen()) close();
	if (binary)
		stream.open(filename.c_str(),
			(isWrite()? std::ios::out : std::ios::in | std::ios::binary));
	else
		stream.open(filename.c_str(),
			(isWrite()? std::ios::out : std::ios::in));
	return isOpen();
} // open

/* --- open --- */
bool Stream::open(const std::string& fn, IO io, bool bin)
{
	filename = fn;
	return open(io, bin);
} // open

/* --- close --- */
void Stream::close()
{
	if (isOpen()) {
		if (isWrite())
			if (header && !footer) writeFooter();
		stream.close();
	}
} // close

/** skipLine */
void Stream::skipLine()
{
	int c;
	do {
		c = stream.get();
	} while (c!='\n' && c!=-1);
} // skipLine

/* --- write --- */
void Stream::write()
{
	assert(isWrite());
	if (!header)
		writeHeader();
} // write

/* --- writeHeader --- */
void Stream::writeHeader()
{
	assert(isWrite());
	assert(isOpen());
	assert(header==false);

	header = true;
} // writeHeader

/* --- writeFooter --- */
void Stream::writeFooter()
{
	assert(isWrite());
	assert(footer==false);

	footer = true;
} // writeFooter
