/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 */

#include "latticelayer.h"
#include "viewer.h"

/** draw */
void LatticeLayer::draw(Painter& painter)
{
	int	H = painter.height();	// save variables that might change
	int	W = painter.width();	// during the scanning...
	Ray	ray;
	VZone*	repZone=NULL;		// replica zone

	//Vector d = view().rayDirection((1.0,0.0);
	double dx = view().uv2dx(1.0,0.0);
	double dy = view().uv2dy(1.0,0.0);
	double dz = view().uv2dz(1.0,0.0);

	GeometryEngine* eng = kernel.engine();
	dword* ptr = painter.data();
	for (int j=0; j<H; j++) {
		if (stop()) break;
		double v = view().jc2v(j);
		for (int i=0; i<W; i++, ptr++) {
			if (*ptr!=geometry.latticeColor && *ptr!=viewer.lattice.hashColor)
				continue;
			double u = view().ic2u(i);

			/* scan to find end of lattice row */
			dword* ps = ptr;
			int is = i;
			for (; i<W; i++, ptr++)
				if (*ptr!=geometry.latticeColor && *ptr!=viewer.lattice.hashColor)
					break;
			if (i==W) ptr--;	// avoid double counting at the end of the line

			double tmax = view().ic2u(i) - u;
			//double u2 = u + tmax/2.0;

			// Find zone and transformation
			//Point p = view().rayPosition(u,v)
			double x = view().uv2x(u,v);
			double y = view().uv2y(u,v);
			double z = view().uv2z(u,v);

			geometry.lockRead();;
			eng->incBodyCheckId();
			repZone = eng->whereRay(x,y,z, dx,dy,dz, tmax/2.0, repZone);

			if (repZone == NULL) {
				geometry.unlockRead();;
				continue;
			}

			// prepare the ray
			ray.init();
			RaySegment segment;
			segment.pos.set(x, y, z);
			segment.dir.set(dx, dy, dz);
			segment.zone = repZone;
			segment.tmin = SMALL3D;
			segment.tmax = tmax;
			ray.push(segment);
			dword color;
			bool end;
			int stuck=0;	// stuck counter
			do {
				int k;
				if (ray.hitZone() == NULL) {
					// try to recover zone
					ray.segment().zone = eng->whereRay(
								ray.segment().pos,
								ray.segment().dir,
								ray.segment().tmin);
					// Set color to error = red
					color = geometry.errorColor;
				} else
				if (ray.hitRegion()->transparent() && viewer.d3.show)
					color = geometry.transparentColor();
				else
					color = ray.hitRegion()->color();

				dword color2 = Color32::darker(color, level);
				color  |= FLAG_REGION;
				color2 |= FLAG_REGION;

				if (ray.hitZone() != NULL) {
					end = eng->intersectRay(&ray, true);
					k   = view().u2ic(u+ray.T())-is;
					// abort if we are stuck at a location due
					// to incorrect transformations
					if (k==0) {
						if (++stuck > 100) end = true;
					} else
						stuck = 0;
				} else {
					end = true;
					k   = i-is;
				}
				while (k>0 and ps<ptr) {
					*ps = (*ps==viewer.lattice.hashColor)?color2:color;
					ps++;
					is++;
					k--;
				}
			} while (!end and ps<ptr);
			geometry.unlockRead();;
			// fill remaining pixels
			while (ps<ptr) *ps++ = color;
		}
	}
} // draw
